/******************************************************************************
 * Copyright (C) 2010-2011 Lastrix                                            *
 *                                                                            *
 * This file is part of JSONSerialization.                                    *
 *                                                                            *
 * JSONSerialization is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by       *
 * the Free Software Foundation, either version 2 of the License, or          *
 * (at your option) any later version.                                        *
 *                                                                            *
 * JSONSerialization is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               *
 * GNU General Public License for more details.                               *
 *                                                                            *
 * You should have received a copy of the GNU General Public License          *
 * along with JSONSerialization. If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************/

package org.lastrix.JSONSerialization.tool;

import org.lastrix.JSONSerialization.IdentifiableProvider;
import org.lastrix.JSONSerialization.exception.LoadingFailedException;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.List;

/**
 * PlainJSONLoader to handle plain files.
 *
 * @author: lastrix
 * Date: 8/10/11
 * Time: 6:04 PM
 * TODO: think better about test functionality and etc, to be sure, that it could give more information.
 */
public class PlainJSONLoader<T> extends AbstractJSONLoader<T> {

	/**
	 * Source file
	 */
	protected final File source;

	/**
	 * Create default JSONLoader. You can not use #load(IdentifiableProvider).
	 */
	public PlainJSONLoader() {
		source = null;
	}

	/**
	 * Create loader for filename (File would be created)
	 *
	 * @param fileName - the file name
	 *
	 * @throws IllegalArgumentException - if file can not be read.
	 * @see #PlainJSONLoader(File)
	 */
	public PlainJSONLoader(final String fileName) throws IllegalArgumentException {
		this(new File(fileName));
	}

	/**
	 * Create loader for file
	 *
	 * @param source - the file
	 *
	 * @throws IllegalArgumentException - if file can not be read.
	 */
	public PlainJSONLoader(final File source) throws IllegalArgumentException {
		if (!source.exists() || !source.isFile() || !source.canRead()) {
			throw new IllegalArgumentException("Specified file '" + source + "' cann't be read.");
		}
		this.source = source;
	}


	@Override
	public List load(final IdentifiableProvider<T> provider) throws LoadingFailedException {
		if (source == null) {
			throw new LoadingFailedException("'Source' is null.");
		}
		try {
			FileReader r = new FileReader(source);
			BufferedReader br = new BufferedReader(r);
			final List list = deserialize(br, provider);
			br.close();
			return list;
		} catch (Exception e) {
			throw new LoadingFailedException(e);
		}
	}
}
