/******************************************************************************
 * Copyright (C) 2010-2011 Lastrix                                            *
 *                                                                            *
 * This file is part of JSONSerialization.                                    *
 *                                                                            *
 * JSONSerialization is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by       *
 * the Free Software Foundation, either version 2 of the License, or          *
 * (at your option) any later version.                                        *
 *                                                                            *
 * JSONSerialization is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               *
 * GNU General Public License for more details.                               *
 *                                                                            *
 * You should have received a copy of the GNU General Public License          *
 * along with JSONSerialization. If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************/

package org.lastrix.JSONSerialization.tool;

import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import org.lastrix.JSONSerialization.tool.validation.AbstractJSONValidationBlock;

import java.io.*;
import java.util.HashMap;
import java.util.Map;

/**
 * @author: lastrix
 * Date: 8/11/11
 * Time: 11:12 AM
 */
public class Validator {
	public static final String JSON = "json";
	protected InputStream schema;
	protected Object root = null;
	protected String schemaVersion;

	public final static String                                    SCHEMA_VERSION = "schemaVersion";
	private             int                                       nextId         = 0;
	private final       Map<Integer, AbstractJSONValidationBlock> blocks         = new HashMap<Integer, AbstractJSONValidationBlock>();

	public Validator(final String schemaName) throws FileNotFoundException, IOException, ParseException {
		this(new FileInputStream(schemaName));
	}

	public Validator(final InputStream schema) throws IOException, ParseException {
		this.schema = schema;
		compile();
	}

	private void compile() throws IOException, ParseException {
		//TODO: make implementation
		JSONParser parser = new JSONParser();
		Object o = parser.parse(new InputStreamReader(schema));
		if (!(o instanceof JSONObject)) {
			throw new IllegalStateException("Invalid schema.");
		}

		JSONObject root = (JSONObject) o;
		schemaVersion = getString(root, SCHEMA_VERSION);
		JSONObject json = getJSONObject(root, JSON);

	}

	/**
	 * Return json object, if speicifed jsonObject doesn't contain valid JSONObject with <code>tag</code>, return empty JSONObject
	 *
	 * @param jsonObject - the json object that hold values
	 * @param tag        - the tag
	 *
	 * @return JSONObject
	 */
	public static JSONObject getJSONObject(final JSONObject jsonObject, final String tag) {
		Object o = jsonObject.get(tag);
		if (!(o instanceof JSONObject)) {
			return new JSONObject();
		}
		return (JSONObject) o;
	}

	/**
	 * Safely return string value
	 *
	 * @param jsonObject
	 * @param tag
	 *
	 * @return String value
	 */
	public static String getString(final JSONObject jsonObject, final String tag) {
		Object o = jsonObject.get(tag);
		if (o == null) {
			return "(null)";
		}
		return o.toString();
	}

	/**
	 * Validate specified text
	 *
	 * @param text - the text
	 *
	 * @return true if text validates, false otherwise
	 */
	public boolean validate(final String text) {
		//TODO: implement this
		return false;
	}

	/**
	 * Add new block, adjust an id to it.
	 *
	 * @param block
	 */
	public void addBlock(AbstractJSONValidationBlock block) {
		block.setId(nextId++);
		blocks.put(block.getId(), block);
	}

	/**
	 * Remove block
	 *
	 * @param block
	 */
	public void removeBlock(AbstractJSONValidationBlock block) {
		blocks.remove(block.getId());
	}
}
