/******************************************************************************
 * Copyright (C) 2010-2011 Lastrix                                            *
 *                                                                            *
 * This file is part of JSONSerialization.                                    *
 *                                                                            *
 * JSONSerialization is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by       *
 * the Free Software Foundation, either version 2 of the License, or          *
 * (at your option) any later version.                                        *
 *                                                                            *
 * JSONSerialization is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               *
 * GNU General Public License for more details.                               *
 *                                                                            *
 * You should have received a copy of the GNU General Public License          *
 * along with JSONSerialization. If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************/

package org.lastrix.JSONSerialization;

import junit.framework.TestCase;
import org.junit.Test;
import org.lastrix.JSONSerialization.tool.*;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author: lastrix
 * Date: 8/13/11
 * Time: 12:11 PM
 */
public class LoaderSaverTest extends TestCase {

	@Test
	public void testZIP() {
		final File temp = createTemp();
		doSaveLoad(new ZIPJSONLoader(temp), new ZIPJSONSaver(temp));
	}

	@Test
	public void testPlain() {
		final File temp = createTemp();
		doSaveLoad(new PlainJSONLoader(temp), new PlainJSONSaver(temp));
	}

	private final File createTemp() {
		File file = null;
		try {
			file = File.createTempFile("jsontests_", "");
		} catch (final IOException e1) {
			assertTrue("Can not create tmp file.", false);
		}
		file.deleteOnExit();
		return file;
	}

	private final void doSaveLoad(final AbstractJSONLoader jsonLoader, final AbstractJSONSaver jsonSaver) {
		final List objects = new ArrayList();
		for (int i = 0; i < 10; i++) {
			objects.add(new SimpleTestClass(new TestKey(i)));
		}

		try {
			jsonSaver.save(objects);
		} catch (final Exception e) {
			e.printStackTrace();
			assertTrue("Cann't write to file.", false);
		}

		List objects2 = null;

		try {
			objects2 = jsonLoader.load();
		} catch (final Exception e) {
			assertTrue("Cann't read from file.", false);
		}

		assertTrue("Lists should be equal.", objects.equals(objects2));
	}
}
