/******************************************************************************
 * Copyright (C) 2010-2011 Lastrix                                            *
 *                                                                            *
 * This file is part of JSONSerialization.                                    *
 *                                                                            *
 * JSONSerialization is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by       *
 * the Free Software Foundation, either version 2 of the License, or          *
 * (at your option) any later version.                                        *
 *                                                                            *
 * JSONSerialization is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               *
 * GNU General Public License for more details.                               *
 *                                                                            *
 * You should have received a copy of the GNU General Public License          *
 * along with JSONSerialization. If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************/

package org.lastrix.JSONSerialization;

import junit.framework.TestCase;
import org.junit.Test;
import org.lastrix.JSONSerialization.exception.LoadingFailedException;
import org.lastrix.JSONSerialization.exception.SavingFailedException;
import org.lastrix.JSONSerialization.tool.PlainJSONLoader;
import org.lastrix.JSONSerialization.tool.PlainJSONSaver;
import org.lastrix.JSONSerialization.tool.ZIPJSONLoader;
import org.lastrix.JSONSerialization.tool.ZIPJSONSaver;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author lastrix
 */
@SuppressWarnings("unused")
public class SerializationTest extends TestCase {

	@Test
	public void testSerialization() {
		File file = createTemp();
		final Map<TestKey, Identifiable<TestKey>> objects = new HashMap<TestKey, Identifiable<TestKey>>();
		for (int i = 1; i < 4; i++) {
			final TestKey k = new TestKey(i);
			objects.put(k, new TestObject(k, i));
		}
		final TestObject to = new TestObject(new TestKey(0), 100);
		to.putAll(objects);
		objects.put(to.getId(), to);
		final Identifiable<TestKey> object = new SimpleTestClass(new TestKey(100500));
		objects.put(object.getId(), object);


		PlainJSONSaver saver = new PlainJSONSaver(file);
		PlainJSONLoader loader = new PlainJSONLoader(file);

		try {
			saver.save(objects.values());
		} catch (final Exception e) {
			assertTrue("Can not write to file.", false);
		}

		final Map<Object, Object> objects2 = new HashMap<Object, Object>();


		try {
			List list = loader.load();
			for (Object o : list) {
				if (o instanceof Identifiable) {
					objects2.put(((Identifiable) o).getId(), o);
				} else {
					fail("An unexpected object found.");
				}
			}
		} catch (final Exception e) {
			e.printStackTrace();
			assertTrue("Can not read from file.", false);
		}
		try {
			final String testStr = objects2.toString();
		} catch (final Exception e) {
			assertTrue("Deserialization failed.", false);
		}
		TestObject o = (TestObject) objects2.get(new TestKey(0));
		assertNotNull("Object shouldn't be null.", o);
		boolean good = true;
		for (Object obj : o.getObjs().values()) {
			if (!objects2.containsValue(obj)) {
				good = false;
				break;
			}
		}
		assertTrue("Not good.", good);
	}

	private static File createTemp() {
		File file = null;
		try {
			file = File.createTempFile("jsontests_", "");
		} catch (final IOException e1) {
			assertTrue("Cann't create tmp file.", false);
		}
		//file.deleteOnExit();
		return file;
	}

	/**
	 * Run test with many-many-many objects in list. This could take a while. 40 sec on Intel E8400 with 8GB RAM
	 */
	@Test
	public void _testManyObjects() {
		File file = createTemp();

		List list = new ArrayList();
		for (int i = 0; i < 500000; i++) {
			list.add(new TestObject(new TestKey(i), i));
		}

		ZIPJSONSaver saver = new ZIPJSONSaver(file);
		try {
			saver.save(list);
		} catch (SavingFailedException e) {
			fail("Saving failed!");
		}

		ZIPJSONLoader<TestKey> loader = new ZIPJSONLoader<TestKey>(file);
		try {
			final List result = loader.load();
			assertTrue("Sizes are not equal.", result.size() == list.size());
		} catch (LoadingFailedException e) {
			fail("Loading failed!");
		}
	}
}
