/******************************************************************************
 * Copyright (C) 2010-2011 Lastrix                                            *
 *                                                                            *
 * This file is part of JSONSerialization.                                    *
 *                                                                            *
 * JSONSerialization is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by       *
 * the Free Software Foundation, either version 2 of the License, or          *
 * (at your option) any later version.                                        *
 *                                                                            *
 * JSONSerialization is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               *
 * GNU General Public License for more details.                               *
 *                                                                            *
 * You should have received a copy of the GNU General Public License          *
 * along with JSONSerialization. If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************/

package org.lastrix.JSONSerialization;

import java.util.HashMap;
import java.util.Map;

/**
 * @author lastrix
 */
@SuppressWarnings("unused")
public class TestObject implements Identifiable<TestKey> {

	private class NestedClassTest {
		int    a;
		String c;

		/**
		 * Generated constructor
		 */
		protected NestedClassTest() {
			super();
		}

		/**
		 * Generated constructor
		 *
		 * @param a - a param
		 * @param c - c param
		 */
		public NestedClassTest(final int a, final String c) {
			super();
			this.a = a;
			this.c = c;
		}

	}

	private enum TestEnum {
		Enum1,
		Enum2,
		Enum3
	}

	private int                                 a    = 0;
	private TestKey                             id   = null;
	private Map<TestKey, Identifiable<TestKey>> objs = new HashMap<TestKey, Identifiable<TestKey>>();
	private NestedClassTest nct;
	private TestEnum        enumTest;

	/**
	 * Generated constructor
	 */
	public TestObject() {
	}

	/**
	 * Generated constructor
	 *
	 * @param id - the id
	 * @param a  - the a param
	 */
	public TestObject(final TestKey id, final int a) {
		super();
		this.id = id;
		this.a = a;
		this.nct = new NestedClassTest(a, id.toString());
		this.enumTest = TestEnum.values()[a % TestEnum.values().length];
	}

	/**
	 * Returns a
	 *
	 * @return an int
	 */
	public final int getA() {
		return this.a;
	}

	/*
	 * (non-Javadoc)
	 * @see org.lastrix.JSONSerialization.Identifiable#getId()
	 */
	@Override
	public TestKey getId() {
		return this.id;
	}

	public void putAll(final Map<TestKey, Identifiable<TestKey>> objs) {
		this.objs.putAll(objs);
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return String.format("(id=%s, a=%d, objs=%s)", this.getId().toString(), this.getA(), this.objs.toString());
	}

	@Override
	public void setId(TestKey id) {
		this.id = id;
	}


	public Map<TestKey, Identifiable<TestKey>> getObjs() {
		return objs;
	}
}
